unit AJB_ADOReg;

interface
uses Classes, Windows, TypInfo, DesignIntf, DesignEditors, AJB_ADO, inifiles,
  SysUtils, ADODB, Dialogs, SQLEdit, AJB_ADOX, registry;
type

  TAJBCommandTextProperty = class(TStringProperty)
  private
    FCommandType: TCommandType;
    FConnection: TADOConnection;
  protected
  public
    procedure Activate; override;
    function AutoFill: Boolean; override;
    procedure Edit; override;
    procedure EditSQLText; virtual;
    function GetAttributes: TPropertyAttributes; override;
    function GetConnection(Opened: Boolean): TADOConnection;
    procedure GetValueList(List: TStrings); virtual;
    procedure GetValues(Proc: TGetStrProc); override;
    property CommandType: TCommandType read FCommandType write FCommandType;
  end;

  TAJBADOTableNameProperty = class(TAJBCommandTextProperty)
  public
    procedure Activate; override;
  end;

  TAJBFilenameProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TADODatabaseProperty = class(TAJBFilenameProperty)
  private
    FDirDlg: Boolean;
    FExt: TFilterType;
  public
    procedure Edit; override;
    procedure Activate; override;
  end;

  TDriverPropEdit = class(TStringProperty)
  private
  public
    procedure GetValues(Proc: TGetStrProc); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TISAMPropEdit = class(TStringProperty)
  private
  public
    procedure GetValues(Proc: TGetStrProc); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TProviderPropEdit = class(TStringProperty)
  private
  public
    procedure GetValues(Proc: TGetStrProc); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure Register;

procedure GetODBCDriverNames(sList: TStringList);

procedure GetISAMNames(sList: TStringList);

implementation
{$R AJB_ADO.dcr}
{$R AJB_ADOX.dcr}

uses FileCtrl;

const
  sOpenFileTitle = 'Open Import File';
  noISAM = '(Access)';

procedure Register;
begin
  RegisterComponents('ADO', [TAJB_ADOConnection, TODBC_Connection, TADOXCatalog, TADOXTableTemplate]);
  RegisterPropertyEditor(TypeInfo(TProvstring), TAJB_ADOConnection, 'Provider', TProviderPropEdit);
  RegisterPropertyEditor(TypeInfo(TProvstring), TADOXCatalog, 'Provider', TProviderPropEdit);
  RegisterPropertyEditor(TypeInfo(TDriverstring), TODBC_Connection, 'ODBCDriver', TDriverPropEdit);
  RegisterPropertyEditor(TypeInfo(TISAMstring), TAJB_ADOConnection, 'ISAM', TISAMPropEdit);

  //register new property editor (shows Linked tables in property)
  RegisterPropertyEditor(TypeInfo(WideString), TADODataSet, 'CommandText', TAJBCommandTextProperty);
  RegisterPropertyEditor(TypeInfo(WideString), TADOTable, 'TableName', TAJBADOTableNameProperty);
  RegisterPropertyEditor(TypeInfo(WideString), TADOCommand, 'CommandText', TAJBCommandTextProperty);
  RegisterPropertyEditor(TypeInfo(TFilename), TAJB_ADOConnection, 'Database', TADODatabaseProperty);
  RegisterPropertyEditor(TypeInfo(TFilename), TADOXCatalog, 'Database', TADODatabaseProperty);

end;

procedure GetODBCDriverNames(sList: TStringList);
//looks up c:\Windows\odbcinst.ini
var
  inifile: TIniFile;
  i, j: Integer;
begin
  inifile := TIniFile.Create('odbcinst.ini');
  try
    inifile.ReadSection('ODBC 32 bit Drivers', sList);
    //remove '(32 bit)' from each driver
    for i := 0 to sList.Count - 1 do
      begin
        j := Pos('(32 bit)', sList[i]);
        if (j > 0) then
          sList[i] := Trim(Copy(sList[i], 1, j - 1));
      end;
  finally
    inifile.Free;
  end;
end;

function EditFileName(ADataSet: TADODataSet; LoadData: Boolean): Boolean;
begin
  with TOpenDialog.Create(nil) do
  try
    Title := sOpenFileTitle;
    DefaultExt := 'adtg';
    Filter := 'Advanced Data Tag format|*.adtg|XML format|*.xml|All files (*.*)|*.*';
    Result := Execute;
    if Result then
      if LoadData then
        ADataSet.LoadFromFile(FileName)
      else
        ADataSet.CommandText := FileName;
  finally
    Free;
  end;
end;

procedure GetISAMNames(sList: TStringList);
var
  reg: TRegistry;
begin
  sList.Clear;
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_LOCAL_MACHINE;
    reg.OpenKey('\Software\Microsoft\Jet\4.0\ISAM Formats', False);
    reg.GetKeyNames(sList);
    reg.CloseKey;
  finally
    reg.Free;
    sList.Insert(0, noISAM);
  end;
end;

procedure TAJBCommandTextProperty.Activate;
var
  PropInfo: PPropInfo;
  Component: TComponent;
begin
  Component := GetComponent(0) as TComponent;
  PropInfo := TypInfo.GetPropInfo(Component.ClassInfo, 'CommandType'); { do not localize }
  if Assigned(PropInfo) then
    CommandType := TCommandType(GetOrdProp(Component, PropInfo))
  else
    CommandType := cmdText;
end;

function TAJBCommandTextProperty.AutoFill: Boolean;
var
  Connection: TADOConnection;
begin
  Connection := GetConnection(False);
  Result := Assigned(Connection) and Connection.Connected;
end;

procedure TAJBCommandTextProperty.Edit;
begin
  case CommandType of
    cmdText, cmdUnknown: EditSQLText;
    cmdFile: EditFileName(GetComponent(0) as TADODataSet, False);
    else
      inherited;
  end;
end;

procedure TAJBCommandTextProperty.EditSQLText;
var
  Command: string;
  Connection: TADOConnection;
begin
  if paDialog in GetAttributes then
    begin
      Command := GetStrValue;
      Connection := GetConnection(True);
      try
        if Connection is TAJB_ADOConnection then
          begin
            if EditSQL(Command, TAJB_ADOConnection(Connection).GetTableNames,
              TAJB_ADOConnection(Connection).GetFieldNames) then
              SetStrValue(Command);
          end
        else if EditSQL(Command, Connection.GetTableNames, Connection.GetFieldNames) then
          SetStrValue(Command);
      finally
        FConnection.Free;
        FConnection := nil;
      end;
    end;
end;

function TAJBCommandTextProperty.GetAttributes: TPropertyAttributes;
begin
  if CommandType in [cmdTable, cmdTableDirect, cmdStoredProc] then
    Result := [paValueList, paSortList, paMultiSelect]
  else {Drop down list for name list}
    Result := [paMultiSelect, paRevertable, paDialog]; {SQL or File}
end;

function TAJBCommandTextProperty.GetConnection(Opened: Boolean): TADOConnection;
var
  Component: TComponent;
  ConnectionString: string;
begin
  Component := GetComponent(0) as TComponent;
  Result := TObject(GetOrdProp(Component, TypInfo.GetPropInfo(Component.ClassInfo,
    'Connection'))) as TADOConnection; { do not localize }
  if not Opened then
    Exit;
  if not Assigned(Result) then
    begin
      ConnectionString := TypInfo.GetStrProp(Component,
        TypInfo.GetPropInfo(Component.ClassInfo, 'ConnectionString')); { do not localize }
      if ConnectionString = '' then
        Exit;
      FConnection := TADOConnection.Create(nil);
      FConnection.ConnectionString := ConnectionString;
      FConnection.LoginPrompt := False;
      Result := FConnection;
    end;
  Result.Open;
end;

procedure TAJBCommandTextProperty.GetValueList(List: TStrings);
var
  Connection: TADOConnection;
begin
  Connection := GetConnection(True);
  if Assigned(Connection) then
  try
    case CommandType of
      cmdTable, cmdTableDirect:
        if Connection is TAJB_ADOConnection then
          TAJB_ADOConnection(Connection).GetTableNames(List)
        else
          Connection.GetTableNames(List);
      cmdStoredProc:
        Connection.GetProcedureNames(List);
    end;
  finally
    if Assigned(FConnection) then
      FConnection.Free;
    FConnection := nil;
  end;
end;

procedure TAJBCommandTextProperty.GetValues(Proc: TGetStrProc);
var
  sl: TStringList;
  i: Integer;
begin
  sl := TStringList.Create;
  try
    GetValueList(sl);
    for i := 0 to sl.Count - 1 do
      Proc(sl[i]);
  finally
    sl.Free;
  end;
end;

procedure TAJBADOTableNameProperty.Activate;
begin
  CommandType := cmdTable;
end;

procedure TAJBFilenameProperty.Edit;
var
  dlgFileOpen: TOpenDialog;
begin
  dlgFileOpen := TOpenDialog.Create(nil);
  dlgFileOpen.Filename := GetValue;
  dlgFileOpen.Options := [ofFileMustExist, ofCreatePrompt, ofHideReadOnly];
  dlgFileOpen.Title := 'Choose a file';
  try
    if dlgFileOpen.Execute then SetValue(dlgFileOpen.Filename);
  finally
    dlgFileOpen.Free;
  end;
end;

function TAJBFilenameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paRevertable];
end;

procedure TADODatabaseProperty.Activate;
var
  PropInfo: PPropInfo;
  Component: TComponent;
  x: TISAMstring;
  s: string;
begin
  FExt := fltAccess;
  FDirDlg := False;
  Component := GetComponent(0) as TComponent;
  PropInfo := TypInfo.GetPropInfo(Component.ClassInfo, 'Connected'); { do not localize }
  if Assigned(PropInfo) then SetOrdProp(Component, PropInfo, Ord(False));
  PropInfo := TypInfo.GetPropInfo(Component.ClassInfo, 'ISAM'); { do not localize }
  if Assigned(PropInfo) then
    begin
      x := TISAMstring(GetStrProp(Component, PropInfo));
      s := Copy(x, 1, 5);
      if CompareText(s, 'dBase') = 0 then
        FDirDlg := True;
      if comparetext(s, 'Excel') = 0 then
        FExt := fltExcel;
      if CompareText(s, 'Lotus') = 0 then
        FExt := fltLotus;
      if CompareText(s, 'Parad') = 0 then
        FDirDlg := True;
      if CompareText(s, 'Text') = 0 then
        FExt := fltText;
    end
  else
    FDirDlg := false;
end;

procedure TADODatabaseProperty.Edit;
var
  dlgFileOpen: TOpenDialog;
  d: string;
begin
  if FDirDlg then
    begin
      if SelectDirectory('Choose Path for Datafiles', '', d) then
        SetValue(d);
    end
  else
    begin
      dlgFileOpen := TOpenDialog.Create(nil);
      dlgFileOpen.Filename := GetValue;
      case FExt of
        fltAccess: dlgFileOpen.Filter := 'Access Databases|*.mdb|All files|*.*';
        fltExcel: dlgFileOpen.Filter := 'Excel Worksheets|*.xl?|All files|*.*';
        fltLotus: dlgFileOpen.Filter := 'Lotus Worksheets|*.wk?|All files|*.*';
        fltText: dlgFileOpen.Filter := 'Text Files|*.txt|All files|*.*';
        else
          dlgFileOpen.Filter := 'All files|*.*';
      end;
      dlgFileOpen.Options := [ofFileMustExist, ofCreatePrompt, ofHideReadOnly];
      dlgFileOpen.Title := 'Choose a file';
      try
        if dlgFileOpen.Execute then SetValue(dlgFileOpen.Filename);
      finally
        dlgFileOpen.Free;
      end;
    end;
end;

function TDriverPropEdit.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TDriverPropEdit.GetValues(Proc: TGetStrProc);
var
  sl: TStringList;
  i: Integer;
begin
  sl := TStringList.Create;
  try
    GetODBCDriverNames(sl);
    sl.Sort;
    for i := 0 to sl.Count - 1 do
      Proc(sl[i]);
  finally
    sl.Free;
  end;
end;

function TISAMPropEdit.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TISAMPropEdit.GetValues(Proc: TGetStrProc);
var
  sl: TStringList;
  i: Integer;
begin
  sl := TStringList.Create;
  try
    GetISAMNames(sl);
    for i := 0 to sl.Count - 1 do
      Proc(sl[i]);
  finally
    sl.Free;
  end;
end;

function TProviderPropEdit.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TProviderPropEdit.GetValues(Proc: TGetStrProc);
var
  sl: TStringList;
  i: Integer;
begin
  sl := TStringList.Create;
  try
    GetProviderNames(sl);
    sl.Sort;
    for i := 0 to sl.Count - 1 do
      Proc(sl[i]);
  finally
    sl.Free;
  end;
end;

end.

